@@ -13,7 +13,9 @@
 #include <sys/times.h>
 #include <errno.h>
 #include <reent.h>
+#include <signal.h>
 #include <unistd.h>
+#include <sys/wait.h>
 #include "swi.h"
 
 /* Forward prototypes.  */
@@ -99,7 +101,7 @@ static inline int
 do_AngelSWI (int reason, void * arg)
 {
   int value;
-  asm volatile ("mov r0, %1; mov r1, %2; swi %a3; mov %0, r0"
+  asm volatile ("mov r0, %1; mov r1, %2; " AngelSWIInsn " %a3; mov %0, r0"
        : "=r" (value) /* Outputs */
        : "r" (reason), "r" (arg), "i" (AngelSWI) /* Inputs */
        : "r0", "r1", "r2", "r3", "ip", "lr", "memory", "cc"
@@ -463,9 +465,15 @@ _kill (int pid, int sig)
 {
   (void)pid; (void)sig;
 #ifdef ARM_RDI_MONITOR
-  /* Note: Both arguments are thrown away.  */
-  return do_AngelSWI (AngelSWI_Reason_ReportException,
-		      (void *) ADP_Stopped_ApplicationExit);
+  /* Note: The pid argument is thrown away.  */
+  switch (sig) {
+	  case SIGABRT:
+		  return do_AngelSWI (AngelSWI_Reason_ReportException,
+				  (void *) ADP_Stopped_RunTimeError);
+	  default:
+		  return do_AngelSWI (AngelSWI_Reason_ReportException,
+				  (void *) ADP_Stopped_ApplicationExit);
+  }
 #else
   asm ("swi %a0" :: "i" (SWI_Exit));
 #endif
@@ -560,7 +568,10 @@ int
 _unlink (const char *path)
 {
 #ifdef ARM_RDI_MONITOR
-  return do_AngelSWI (AngelSWI_Reason_Remove, &path);
+  int block[2];
+  block[0] = path;
+  block[1] = strlen(path);
+  return wrap (do_AngelSWI (AngelSWI_Reason_Remove, block)) ? -1 : 0;
 #else
   (void)path;
   asm ("swi %a0" :: "i" (SWI_Remove));
@@ -631,11 +642,14 @@ _times (struct tms * tp)
 int
 _isatty (int fd)
 {
+  int fh = remap_handle (fd);
 #ifdef ARM_RDI_MONITOR
-  return do_AngelSWI (AngelSWI_Reason_IsTTY, &fd);
+  return wrap (do_AngelSWI (AngelSWI_Reason_IsTTY, &fh));
 #else
-  (void)fd;
-  asm ("swi %a0" :: "i" (SWI_IsTTY));
+  asm ("mov r0, %1; swi %a0"
+       : /* No outputs */
+       : "i" (SWI_IsTTY), "r"(fh)
+       : "r0");
 #endif
 }
 
@@ -643,7 +657,28 @@ int
 _system (const char *s)
 {
 #ifdef ARM_RDI_MONITOR
-  return do_AngelSWI (AngelSWI_Reason_System, &s);
+  int block[2];
+  int e;
+
+  /* Hmmm.  The ARM debug interface specification doesn't say whether
+     SYS_SYSTEM does the right thing with a null argument, or assign any
+     meaning to its return value.  Try to do something reasonable....  */
+  if (!s)
+    return 1;  /* maybe there is a shell available? we can hope. :-P */
+  block[0] = s;
+  block[1] = strlen (s);
+  e = wrap (do_AngelSWI (AngelSWI_Reason_System, block));
+  if ((e >= 0) && (e < 256))
+    {
+      /* We have to convert e, an exit status to the encoded status of
+         the command.  To avoid hard coding the exit status, we simply
+	 loop until we find the right position.  */
+      int exit_code;
+
+      for (exit_code = e; e && WEXITSTATUS (e) != exit_code; e <<= 1)
+	continue;
+    }
+  return e;
 #else
   (void)s;
   asm ("swi %a0" :: "i" (SWI_CLI));
@@ -654,8 +689,12 @@ int
 _rename (const char * oldpath, const char * newpath)
 {
 #ifdef ARM_RDI_MONITOR
-  const char *block[2] = {oldpath, newpath};
-  return do_AngelSWI (AngelSWI_Reason_Rename, block);
+  int block[4];
+  block[0] = oldpath;
+  block[1] = strlen(oldpath);
+  block[2] = newpath;
+  block[3] = strlen(newpath);
+  return wrap (do_AngelSWI (AngelSWI_Reason_Rename, block)) ? -1 : 0;
 #else
   (void)oldpath; (void)newpath;
   asm ("swi %a0" :: "i" (SWI_Rename));
