@@ -26,6 +26,15 @@
 #include <sys/lock.h>
 #include "local.h"
 
+#ifdef _REENT_SMALL
+const struct __sFILE_fake __sf_fake_stdin =
+    {_NULL, 0, 0, 0, 0, {_NULL, 0}, 0, _NULL};
+const struct __sFILE_fake __sf_fake_stdout =
+    {_NULL, 0, 0, 0, 0, {_NULL, 0}, 0, _NULL};
+const struct __sFILE_fake __sf_fake_stderr =
+    {_NULL, 0, 0, 0, 0, {_NULL, 0}, 0, _NULL};
+#endif
+
 static _VOID
 _DEFUN(std, (ptr, flags, file, data),
             FILE *ptr _AND
