@@ -1,8 +1,3 @@
-/* This file may have been modified by DJ Delorie (Jan 1991).  If so,
-** these modifications are Coyright (C) 1991 DJ Delorie, 24 Kirsten Ave,
-** Rochester NH, 03867-2954, USA.
-*/
-
 /*-
  * Copyright (c) 1988 The Regents of the University of California.
  * All rights reserved.
@@ -29,7 +24,7 @@
 
 int
 _DEFUN (putenv, (str),
-	_CONST char *str)
+	char *str)
 {
   return _putenv_r (_REENT, str);
 }
